package com.lutao.cas;

import cn.hutool.json.JSONObject;
import lombok.*;
import lombok.experimental.Accessors;

import java.io.Serializable;

/**
 * 消息信息
 * 响应信息主体
 *
 * @param <T>
 * @author yx
 * @date 2023/10/20
 */
@Builder
@ToString
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
public class MessageInfo<T> implements Serializable{

	private static final long serialVersionUID = -8703188330984075284L;

	@Getter
	@Setter
	private int code;

	@Getter
	@Setter
	private String msg;

	@Getter
	@Setter
	private T data;

	public static <T> MessageInfo<T> ok(){
		return restResult(null, CommonConstants.SUCCESS, "success");
	}

	public static <T> MessageInfo<T> ok(T data){
		return restResult(data, CommonConstants.SUCCESS, "success");
	}

	public static <T> MessageInfo<T> ok(String data){
		JSONObject jsonObject = new JSONObject();
		jsonObject.putOnce("result",data);
		return (MessageInfo<T>) restResult(jsonObject, CommonConstants.SUCCESS, "success");
	}

	public static <T> MessageInfo<T> ok(T data, String msg){
		return restResult(data, CommonConstants.SUCCESS, msg);
	}

	public static <T> MessageInfo<T> failed(){
		return restResult(null, CommonConstants.FAIL, null);
	}

	public static <T> MessageInfo<T> failed(String msg){
		return restResult(null, CommonConstants.FAIL, msg);
	}

	public static <T> MessageInfo<T> failed(T data){
		return restResult(data, CommonConstants.FAIL, null);
	}

	public static <T> MessageInfo<T> failed(T data, String msg){
		return restResult(data, CommonConstants.FAIL, msg);
	}

	private static <T> MessageInfo<T> restResult(T data, int code, String msg){
		MessageInfo<T> apiResult = new MessageInfo<>();
		apiResult.setCode(code);
		apiResult.setData(data);
		apiResult.setMsg(msg);
		return apiResult;
	}

}
