package com.lutao.cas;

import com.lutao.cas.config.DataSourceConstant;
import org.apereo.cas.authentication.AuthenticationHandlerExecutionResult;
import org.apereo.cas.authentication.PreventedException;
import org.apereo.cas.authentication.UsernamePasswordCredential;
import org.apereo.cas.authentication.handler.support.AbstractUsernamePasswordAuthenticationHandler;
import org.apereo.cas.authentication.principal.PrincipalFactory;
import org.apereo.cas.configuration.CasConfigurationProperties;
import org.apereo.cas.services.ServicesManager;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.stereotype.Component;

import javax.security.auth.login.AccountException;
import javax.security.auth.login.FailedLoginException;
import java.security.GeneralSecurityException;
import java.util.HashMap;

@Component
@EnableConfigurationProperties(CasConfigurationProperties.class)
public class MyAuthenticationHandler extends AbstractUsernamePasswordAuthenticationHandler {

    public MyAuthenticationHandler(String name, ServicesManager servicesManager, PrincipalFactory principalFactory, Integer order) {
        super(name, servicesManager, principalFactory, order);
    }


    @Override
    protected AuthenticationHandlerExecutionResult authenticateUsernamePasswordInternal(UsernamePasswordCredential credential, String originalPassword) throws GeneralSecurityException, PreventedException {
        String username = credential.getUsername();

        String password = credential.getPassword();

        System.out.println(username);
        System.out.println(password);
        // JDBC模板依赖于连接池来获得数据的连接，所以必须先要构造连接池
        DriverManagerDataSource dataSource = new DriverManagerDataSource();
        dataSource.setDriverClassName(DataSourceConstant.DRIVER_CLASS_NAME);
        dataSource.setUrl(DataSourceConstant.URL);
        dataSource.setUsername(DataSourceConstant.USER_NAME);
        dataSource.setPassword(DataSourceConstant.PASSWORD);

        JdbcTemplate jdbcTemplate = new JdbcTemplate();
        jdbcTemplate.setDataSource(dataSource);

        String sql = "select * from sys_user_info where username = ?";

        SysUser info = (SysUser) jdbcTemplate.queryForObject(sql, new Object[]{username}, new BeanPropertyRowMapper(SysUser.class));

        System.out.println(info.getUsername());
        System.out.println(info.getPassword());

        if (info == null) {
            throw new AccountException("Sorry, username not found!");
        }
        PassWordEncoderEncrypt passWordEncoderEncrypt = new PassWordEncoderEncrypt();

        boolean matches = passWordEncoderEncrypt.matches(password,info.getPassword());
        System.out.println(matches);
        if (!matches) {
            throw new FailedLoginException("Sorry, password not correct!");
        } else {

            //可自定义返回给客户端的多个属性信息
            HashMap<String, Object> returnInfo = new HashMap<>();
            returnInfo.put("userId",info.getUserId());
            returnInfo.put("name",info.getUsername());
            returnInfo.put("phone",info.getPhone());
            returnInfo.put("deptId",info.getDeptId());
            returnInfo.put("companyId",info.getCompanyId());
            returnInfo.put("nickname",info.getNickname());
            returnInfo.put("email",info.getEmail());


//            AuthenticationHandlerExecutionResult handlerResult = createHandlerResult(credential,
//                    this.principalFactory.createPrincipal(username, returnInfo), list);
//
//            System.out.println(handlerResult);
            return createHandlerResult(credential,
                    this.principalFactory.createPrincipal(username, returnInfo));
        }

    }
}
