package com.lutao.cas.controller;

import cn.hutool.core.util.ObjectUtil;
import com.lutao.cas.MessageInfo;
import com.lutao.cas.config.DateSourceConfig;
import com.lutao.cas.pojo.entity.PageSysUserService;
import com.lutao.cas.pojo.entity.SysUserService;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/sys-user-service")
public class SysUserServiceController {

    @PostMapping("/add")
    public MessageInfo addSysUserService(@RequestBody SysUserService sysUserService) {
        JdbcTemplate jdbcTemplate = DateSourceConfig.creteJdbcTemplate();
        int update = jdbcTemplate.update("insert into sys_user_service (user_id, service_url, create_time, update_time) values (?, ?, ?, ?)",
                sysUserService.getUserId(), sysUserService.getServiceUrl(), LocalDateTime.now(), LocalDateTime.now());

        if (update > 0) {
            return MessageInfo.ok();
        } else {
            return MessageInfo.failed();
        }
    }

    @PutMapping("/update")
    public MessageInfo updateSysUserService(@RequestBody SysUserService sysUserService) {
        JdbcTemplate jdbcTemplate = DateSourceConfig.creteJdbcTemplate();

        int update = jdbcTemplate.update("update sys_user_service set service_url = ?, update_time = ? where id = ?",
                sysUserService.getServiceUrl(), LocalDateTime.now(), sysUserService.getUserId());

        if (update > 0) {
            return MessageInfo.ok();
        } else {
            return MessageInfo.failed();
        }
    }

    @DeleteMapping("/delete/{id}")
    public MessageInfo deleteSysUserService(@PathVariable("id") Integer id) {
        JdbcTemplate jdbcTemplate = DateSourceConfig.creteJdbcTemplate();

        int update = jdbcTemplate.update("delete from sys_user_service where id = ?", id);

        if (update > 0) {
            return MessageInfo.ok();
        } else {
            return MessageInfo.failed();
        }
    }

    @GetMapping("/get/{id}")
    public MessageInfo<SysUserService> getSysUserService(@PathVariable("id") Integer id) {
        JdbcTemplate jdbcTemplate = DateSourceConfig.creteJdbcTemplate();

        SysUserService sysUserService = jdbcTemplate.queryForObject("select * from sys_user_service where id = ?", new Object[]{id}, (resultSet, i) -> {
            SysUserService sysUserService1 = new SysUserService();
            sysUserService1.setId(resultSet.getInt("id"));
            sysUserService1.setUserId(resultSet.getInt("user_id"));
            sysUserService1.setServiceUrl(resultSet.getString("service_url"));
            sysUserService1.setCreateTime(resultSet.getTimestamp("create_time").toLocalDateTime());
            sysUserService1.setUpdateTime(resultSet.getTimestamp("update_time").toLocalDateTime());
            return sysUserService1;
        });
        return MessageInfo.ok(sysUserService);
    }

    @GetMapping("/getAllPage")
    public MessageInfo<PageSysUserService> getAllSysUserService(Integer current, Integer size, Integer userId) {
        JdbcTemplate jdbcTemplate = DateSourceConfig.creteJdbcTemplate();

        List<SysUserService> query = new ArrayList<>();
        Long total = 0L;
        if (ObjectUtil.isEmpty(userId)) {
            query = jdbcTemplate.query("select * from sys_user_service limit ? , ?", new Object[]{current, size}, new BeanPropertyRowMapper<>(SysUserService.class));
            System.out.println(query);
            total = jdbcTemplate.queryForObject("select count(*) from sys_user_service", Long.class);
        } else {
            query = jdbcTemplate.query("select * from sys_user_service where user_id = ? limit ? , ?", new Object[]{userId, current, size}, new BeanPropertyRowMapper<>(SysUserService.class));
            total = jdbcTemplate.queryForObject("select count(*) from sys_user_service where user_id = ?", new Object[]{userId}, Long.class);

        }
        PageSysUserService pageSysUserService = new PageSysUserService();
        pageSysUserService.setTotal(total);
        pageSysUserService.setList(query);
        return MessageInfo.ok(pageSysUserService);
    }

}
